var KeyHandler;

KeyHandler = (function() {
  var DOWN, LEFT, RIGHT, UP;

  LEFT = 37;

  UP = 38;

  RIGHT = 39;

  DOWN = 40;

  function KeyHandler(tree_widget) {
    this.tree_widget = tree_widget;
    if (tree_widget.options.keyboardSupport) {
      $(document).bind('keydown.jqtree', $.proxy(this.handleKeyDown, this));
    }
  }

  KeyHandler.prototype.deinit = function() {
    return $(document).unbind('keydown.jqtree');
  };

  KeyHandler.prototype.handleKeyDown = function(e) {
    var current_node, key, moveDown, moveLeft, moveRight, moveUp, selectNode;
    if (!this.tree_widget.options.keyboardSupport) {
      return;
    }
    if ($(document.activeElement).is('textarea,input,select')) {
      return true;
    }
    current_node = this.tree_widget.getSelectedNode();
    selectNode = (function(_this) {
      return function(node) {
        if (node) {
          _this.tree_widget.selectNode(node);
          if (_this.tree_widget.scroll_handler && (!_this.tree_widget.scroll_handler.isScrolledIntoView($(node.element).find('.jqtree-element')))) {
            _this.tree_widget.scrollToNode(node);
          }
          return false;
        } else {
          return true;
        }
      };
    })(this);
    moveDown = (function(_this) {
      return function() {
        return selectNode(_this.getNextNode(current_node));
      };
    })(this);
    moveUp = (function(_this) {
      return function() {
        return selectNode(_this.getPreviousNode(current_node));
      };
    })(this);
    moveRight = (function(_this) {
      return function() {
        if (current_node.isFolder() && !current_node.is_open) {
          _this.tree_widget.openNode(current_node);
          return false;
        } else {
          return true;
        }
      };
    })(this);
    moveLeft = (function(_this) {
      return function() {
        if (current_node.isFolder() && current_node.is_open) {
          _this.tree_widget.closeNode(current_node);
          return false;
        } else {
          return true;
        }
      };
    })(this);
    if (!current_node) {
      return true;
    } else {
      key = e.which;
      switch (key) {
        case DOWN:
          return moveDown();
        case UP:
          return moveUp();
        case RIGHT:
          return moveRight();
        case LEFT:
          return moveLeft();
      }
    }
  };

  KeyHandler.prototype.getNextNode = function(node, include_children) {
    var next_sibling;
    if (include_children == null) {
      include_children = true;
    }
    if (include_children && node.hasChildren() && node.is_open) {
      return node.children[0];
    } else {
      if (!node.parent) {
        return null;
      } else {
        next_sibling = node.getNextSibling();
        if (next_sibling) {
          return next_sibling;
        } else {
          return this.getNextNode(node.parent, false);
        }
      }
    }
  };

  KeyHandler.prototype.getPreviousNode = function(node) {
    var previous_sibling;
    if (!node.parent) {
      return null;
    } else {
      previous_sibling = node.getPreviousSibling();
      if (previous_sibling) {
        if (!previous_sibling.hasChildren() || !previous_sibling.is_open) {
          return previous_sibling;
        } else {
          return this.getLastChild(previous_sibling);
        }
      } else {
        if (node.parent.parent) {
          return node.parent;
        } else {
          return null;
        }
      }
    }
  };

  KeyHandler.prototype.getLastChild = function(node) {
    var last_child;
    if (!node.hasChildren()) {
      return null;
    } else {
      last_child = node.children[node.children.length - 1];
      if (!last_child.hasChildren() || !last_child.is_open) {
        return last_child;
      } else {
        return this.getLastChild(last_child);
      }
    }
  };

  return KeyHandler;

})();

module.exports = KeyHandler;
